' bandpass filter example
' shows how to manually plot gain function for time-varying filters
' requires subroutine file freqresp.prg
' for version 5
' 5/28/2003 h

'include subroutine
include freqresp

'load data
%path = @evpath + "\example files\data"
cd %path
load bpf

'log quartery gdp
series lgdp = log(gdp)

'store weights from asymmetric filter
!low = 6
!upr = 32
lgdp.bpf(type=asym,low=!low,high=!upr,detrend=c,lag=12,w=wasy,i1) cyc_asy

'fill frequency vector
!n=100
!pi = @acos(-1)
!delta = !pi/(!n-1)
vector(!n) fq
fq(1)=0
for !i=2 to !n
	fq(!i) = fq(!i-1) + !delta
next

!side=2
'get freq resp at obs 1
!obs=1
vector real01
vector imag01
vector w = @transpose(@rowextract(wasy,!obs))
call freqresp(w, !obs, fq, !side, real01, imag01)
vector gain01
call fr2gain(real01, imag01, gain01)

'get freq resp at obs 10
!obs=10
vector real10
vector imag10
vector w = @transpose(@rowextract(wasy,!obs))
call freqresp(w, !obs, fq, !side, real10, imag10)
vector gain10
call fr2gain(real10, imag10, gain10)

'get freq resp at middle of sample
scalar midobs = @round(@obs(lgdp)/2)
vector realmid
vector imagmid
vector w = @transpose(@rowextract(wasy,midobs))
call freqresp(w, midobs, fq, !side, realmid, imagmid)
vector gainmid
call fr2gain(realmid, imagmid, gainmid)

'convert freq to cycles/period to match eviews graph
vector xaxis = fq/(2*!pi)
'combine frequency response data into matrix
matrix(@rows(fq),8) xy=na
'obs 1 freq resp
colplace(xy,xaxis,1)
colplace(xy,gain01,2)
'obs 10 freq resp
colplace(xy,xaxis,3)
colplace(xy,gain10,4)
'middle obs freq resp
colplace(xy,xaxis,5)
colplace(xy,gainmid,6)
'ideal filter gain
xy(1,7)=0
xy(2,7)=1/!upr
xy(3,7)=1/!upr
xy(4,7)=1/!low
xy(5,7)=1/!low
xy(6,7)=0.5

xy(1,8)=0
xy(2,8)=0
xy(3,8)=1
xy(4,8)=1
xy(5,8)=0
xy(6,8)=0

'plot as xy pair graph
freeze(gra1) xy.xyline(b)
gra1.legend columns(1) position(right)
gra1.addtext(l) Gain
gra1.scale(bottom) range(0,0.5)
gra1.setelem(1) legend("")
gra1.setelem(2) legend("obs 1")
gra1.setelem(3) legend("")
gra1.setelem(4) legend("obs 10")
gra1.setelem(5) legend("")
gra1.setelem(6) legend("obs mid")
gra1.setelem(7) legend("")
gra1.setelem(8) legend("ideal")
gra1.setelem(4) lcolor(150,150,150)
show gra1

