'frequency response function
' w ... (input) vector of weights
'i0 ... (input) integer: w(i0) is the lag 0 weight
' f ... (input) frequency vector to evaluate at
'               usually equally spaced points from 0 to PI
'side ... (input) integer: side=2 if w contains two sided
'         from time-varying filters
'         side=1 if w contains only one-side from
'         time-invariant symmetric filters
'
'real ... (output) vector of @rows(f) that contains the
'         real part of the frequency response function
'imag ... (output) vector of @rows(f) that contains the
'         imaginary part of the frequency response function
'
'example_frf.prg illustrates how to use this subroutine
'to plot customized freq resp functions
subroutine local freqresp(vector w, scalar i0, vector f, scalar side, vector real, vector imag)
'11/11/02 (h) for version 5

	'resize return vectors
	!n = @rows(f)
	vector(!n) real=0
	vector(!n) imag=0
	'real part
	!n = @rows(w)
	for !i=1 to !n
		real = real + w(!i)*cos(f*(!i-i0))
	next
	'imaginary part
	for !i=1 to !n
		imag = imag - w(!i)*sin(f*(!i-i0))
	next

	if (side=1) then
		for !i=2 to !n
			real = real + w(!i)*cos(f*(i0-!i))
		next
		for !i=2 to !n
			imag = imag - w(!i)*sin(f*(i0-!i))
		next
	endif
endsub

'fills gain with amplitude of complex vector given by real and imag
subroutine local fr2gain(vector real, vector imag, vector gain)
'11/11/02 (h) for version 5
	!len = @rows(real)
	vector(!len) gain=na
	for !i=1 to !len
		gain(!i) = real(!i)*real(!i) + imag(!i)*imag(!i)
	next
	gain = @sqrt(gain)
endsub

'fills lead/lad index idx given zero lag index i0
subroutine local fillidx(vector idx, scalar i0)
'11/11/02 (h) for version 5
	!len = @rows(idx)
	for !i=1 to !len
		idx(!i) = !i-i0
	next
endsub
