' CLOGIT1.PRG (1.0 - 6/29/98)
' revised for version 7.0 (11/19/2009)
' example program for EViews LogL object
'
' conditional logit with 3 outcomes, and a test for IIA
'
' for a discussion, see Section 19.7.2 of Greene, William H. (1997) 
' Econometric Analysis, 3rd edition, Prentice-Hall

'change path to program path
%path = @runpath
cd %path

' load artificial data
load mlogit

' full conditional logit using all three outcomes
' declare coef vector to use in MLE
coef(2) a1 = 0.1	' true values 0.6, -0.3
coef(2) b1 = 0.1 	' true values 0.6, 0.7

' for use in derivative expression
series stores = d1*stores1+d2*stores2+d3*stores3
series dist = d1*dist1+d2*dist2+d3*dist3

' specify likelihood for three outcome logit
logl ll1
ll1.append @logl logl1
ll1.append xb1 = a1(1)*stores1+a1(2)*dist1
ll1.append xb2 = a1(1)*stores2+a1(2)*dist2+b1(1)*inc
ll1.append xb3 = a1(1)*stores3+a1(2)*dist3+b1(2)*inc
ll1.append denom = exp(xb1)+exp(xb2)+exp(xb3)
ll1.append logl1 = d1*xb1+d2*xb2+d3*xb3-log(denom)

' analytic derivatives
ll1.append @deriv a1(1) grada1 a1(2) grada2 b1(1) gradb2 b1(2) gradb3
ll1.append grada1 = stores-(stores1*exp(xb1)+stores2*exp(xb2)+stores3*exp(xb3))/denom
ll1.append grada2 = dist-(dist1*exp(xb1)+dist2*exp(xb2)+dist3*exp(xb3))/denom
ll1.append gradb2 = (d2-exp(xb2)/denom)*inc
ll1.append gradb3 = (d3-exp(xb3)/denom)*inc

' estimate by MLE
ll1.ml(showopts,m=1000,c=1e-5)
show ll1.output

' compute predicted probabilities
series y2hat = exp(xb2)/denom
series y3hat = exp(xb3)/denom
series y1hat = 1-y2hat-y3hat

' compute actual and predicted outcome (the one with highest predicted probability)
series y = (d2=1)*2+3*(d3=1)+1*(d2=0 and d3=0)
series yhat = 2*(y2hat>y3hat and y2hat>y1hat) + 3*(y3hat>y2hat and y3hat>y1hat) + 1*(y1hat>=y3hat and y1hat>=y2hat)

' show prediction table
group predict y yhat
show predict.freq

'-------------------------------------------------------------------------------------
' to test IIA restriction estimate model dropping choice 3 (use only outcomes 1 and 2)
'-------------------------------------------------------------------------------------

' initialize params
coef(2) a2 = 0.1
coef(1) b2 = 0.1

' likelihood for restricted model
' make sure not to use the same name for expressions as in ll1
logl ll2
ll2.append @logl logl2
ll2.append xb12 = a2(1)*stores1+a2(2)*dist1
ll2.append xb22 = a2(1)*stores2+a2(2)*dist2+b2(1)*inc
ll2.append denom2 = exp(xb12)+exp(xb22)
ll2.append logl2 = d1*xb12+d2*xb22-log(denom2)

' analytic derivatives
ll2.append @deriv a2(1) grada12 a2(2) grada22 b2(1) gradb22 
ll2.append grada12 = d1*stores1+d2*stores2-(stores1*exp(xb12)+stores2*exp(xb22))/denom2
ll2.append grada22 = d1*dist1+d2*dist2-(dist1*exp(xb12)+dist2*exp(xb22))/denom2
ll2.append gradb22 = (d2-exp(xb22)/denom2)*inc

' estimate excluding choice 3 data
smpl @all if d3=0
ll2.ml(showopts,m=1000,c=1e-5)
show ll2.output

' compute Hausman statistic for IIA test

' get variance of unrestriced
sym var_1 = ll1.@coefcov
' resize to match var_2 (this trick depends on the ordering of the coefs)
sym(3,3) var_1

' get variance of restricted
sym var_2 = ll2.@coefcov

' get coefs
for !i=1 to 2
	matrix(3,1) coef_{!i} 
	coef_{!i}(1,1) = a{!i}(1)
	coef_{!i}(2,1) = a{!i}(2)
	coef_{!i}(3,1) = b{!i}(1)	
next

' compute test statistic
coef cdiff = coef_2 - coef_1
sym vdiff = var_2 - var_1
matrix hs = @transpose(cdiff) * @inverse(vdiff) * cdiff

' display results in table
table out
setcolwidth(out,1,20)
out(1,1) = "Hausman test for I.I.A.:"
out(2,1) = "chi-sqr(" + @str(@rows(cdiff)) + ") = "
out(2,2) = @str(hs(1))
out(2,3) = "p-value"
out(2,4) = 1-@cchisq(hs(1),@rows(cdiff))
show out
