' DISEQ1.PRG 
' checked for version 7.0 (11/19/2009)
' example program for EViews LogL object

' disequilibrium switching model

' Exercise 15.14-15.15 in (page 644-646), Judge, et al. (1985), 
' The Theory and Practice of Econometrics, 2nd Edition, 
' John Wiley & Sons.

' create workfile
wfcreate diseq u 1 21

' load data in exercise 15.14 (Table 15.3)
' one obs extra for presample value
series u1
series v1
series y1
series w1
u1.fill(o=2) 0.41,-0.09,0.39,-0.52,1.37,-0.8,0.63,1.83,-0.85,-0.85,-1.29,-0.46,-0.79,-0.63,-0.62,-1.53,0.61,0.92,-0.26,-0.93
v1.fill(o=2) 0.46,4.78,2.76,-2.83,3.12,0.28,-0.02,2.81,2.41,0.09,-1.37,0.23,-0.45,2.64,3.83,1.23,-0.81,-1.04,3.33,1.92
y1.fill(o=2) 574.21,653.85,733.81,818.29,899.29,972.78,1057.56,1132.86,1215.8,1292.58,1373.82,1459.22,1538.38,1617.47,1692.97,1777.61,1858.21,1939.75,2010.6,2090.36
w1.fill(o=2) 1.69,1.88,1.94,1.96,2.15,2.16,2.29,2.31,2.59,2.67,2.76,2.73,2.94,3.06,3.19,3.17,3.3,3.44,3.46,3.62


' set starting value for p
smpl 1 1
series p1 = 43.21

' generate p1, d1, s1, q1
model m1
m1.append d1 = 60-1.8*p1(-1)+0.05*y1+u1
m1.append s1 = 10+2.5*p1(-1)-3*w1+v1
m1.append q1 = (d1<s1)*d1+(d1>=s1)*s1
m1.append p1 = p1(-1)+0.04*(d1-s1)

smpl 2 21
m1.solve

' rename 
rename d1_0 d1
rename s1_0 s1
rename q1_0 q1
p1 = p1_0		' already exists; overwrite

' declare coef vectors to use in likelihood spec
coef(3) alpha 	' true values 60, -1.8, 0.05
coef(3) beta 	' true values 10, 2.5, -3
coef(1) gamma 	' true value 0.04
coef(2) sigma 	' true values 1, 2
!pi = @acos(-1)

' setup likelihood as in eqs (2.3.16) & (2.3.17) of Quandt, page 32
' assume zero correlation between demand and supply error
series dp_pos = (d(p1)>0)*d(p1)
series dp_neg = (d(p1)<=0)*(-1)*d(p1)
logl ll1
ll1.append @logl logl1
ll1.append ud = q1-alpha(1)-alpha(2)*p1(-1)-alpha(3)*y1+dp_pos/gamma(1)
ll1.append us = q1-beta(1)-beta(2)*p1(-1)-beta(3)*w1+dp_neg/gamma(1)
ll1.append logl1 = -log(2*!pi) -log(@abs(gamma(1))) -log(sigma(1)) -log(sigma(2)) -ud^2/sigma(1)^2/2 -us^2/sigma(2)^2/2

' estimate 2SLS for starting values
equation eqa.tsls q1 c p1(-1) y1 dp_pos @ c p1(-1) y1 w1
alpha = eqa.@coefs
sigma(1) = eqa.@se
show eqa.output

equation eqb.tsls q1 c p1(-1) w1 dp_neg @ c p1(-1) y1 w1
beta = eqb.@coefs
sigma(2) = eqb.@se
show eqb.output
gamma(1) = -1/eqb.c(4)

' do mle and display output
ll1.ml(showopts, m=1000, c=1e-5)
show ll1.output
