' EGARCH1.PRG (1.0 - 6/29/98)
' example program for EViews LogL object
' revised for version 7.0 (11/19/2009)
' Estimate EGARCH with generalized error distribution (Nelson's specification) 
'
' For discussion, see p. 668 of Hamilton, James D. (1994) Time Series Analysis, 
' Princeton University Press.


'change path to data path
%path = @runpath
cd %path

' load workfile
load gerus

series y = 100*dlog(ger)


' set sample to 1/2/82-7/9/92 (10/6/86 is obs 3945)
sample s0 2748 2750
sample s1 2751 5392
smpl s1


' get starting values from Gaussian EGARCH-M
equation eq1
eq1.arch(2,2,egarch,archm=var) y c y(-1)
show eq1.output


' declare and initialize parameters

' coefs on lagged variance
coef(2) delta
delta(1) = eq1.c(8)
delta(2) = eq1.c(9)

' coefs on lagged resids
coef(2) alpha 
alpha(1) = eq1.c(5)
alpha(2) = eq1.c(7)

' coef on asym term
coef(1) chi
chi(1) = eq1.c(6)/eq1.c(5)

' coefs on deterministic terms
coef(2) rho
rho(1) = 2*log(eq1.@se)
rho(2) = 0 

' 0<nu<2 is thick tails; nu>2 is thin tails
coef(1) nu = 2 

!pi = @acos(-1)

' set presample values of expressions in logl
smpl s0
series zeta = log(1+rho(2)*ndays)
series h = exp(rho(1))
series z = 0
series temp1 = @abs(z(-1)) + chi(1)*z(-1)


' set up EGARCH likelihood
' note that E|v{t-1}| is absorbed in overall constant rho(1) 
logl ll1
ll1.append @logl logl
'zeta = unconditional mean of log(ht)
ll1.append zeta = log(1+rho(2)*ndays)
'log(lambda) p.668 last equation
ll1.append loggam1 = @gammalog(1/nu(1))
ll1.append loglam = -log(2)/nu(1) + 0.5*( loggam1 - @gammalog(3/nu(1)) )
ll1.append temp1 = @abs(z(-1)) + chi(1)*z(-1)
ll1.append log(h) = rho(1) + zeta + delta(1)*(log(h(-1))-zeta(-1)) + delta(2)*(log(h(-2))-zeta(-2)) + alpha(1)*temp1 + alpha(2)*temp1(-1)
ll1.append res = y - c(1)*h - c(2) - c(3)*y(-1)
ll1.append z = res/@sqrt(h)
ll1.append logl = log(nu(1)) - loglam - (1+1/nu(1))*log(2) - loggam1 - 0.5*@abs(z/exp(loglam))^nu(1) - 0.5*log(h)


' estimate and display output
smpl s1
ll1.ml(showopts, m=1000, c=1e-5)
show ll1.output
