' GARCH1.PRG (1.0 - 6/29/98)
' example program for EViews LogL object
' revised for version 5.0 (11/19/2009)
'
' GARCH(1,1) model with MA(1) errors and coef restrictions. 
'
' This is the specification described in Section 9.1 (pp. 3014-3017) 
' of Bollerslev, Tim, Robert F. Engle and Daniel B. Nelson (1994) 
' "ARCH Models", in Chapter 49 of the Handbook of Econometrics, 
' Volume 4, North-Holland, applied to different data. 

'change path to program path
%path = @runpath
cd %path

' load workfile
load gerus

series y = 100*dlog(ger)


' set sample (1/2/82-7/9/92) as in Table 1, column 1
' 10/6/86 is obs 3945
sample s0 2750 2750
sample s1 2751 5392
smpl s1


' declare coef vectors to use in ARCH likelihood
coef(1) mu = .1
coef(1) theta = .1
coef(2) omega = .1
coef(1) alpha = .1
coef(1) beta = .1


' get starting values from MA(1)
equation eq_temp.ls y c ma(1)
mu(1) = eq_temp.c(1)
theta(1) = eq_temp.c(2)
omega(1) = eq_temp.@se^2


' set presample values of expressions in logl
smpl s0
series sig2 = omega(1)
series resma = 0


' set up ARCH likelihood
logl ll1
ll1.append @logl logl
ll1.append res = y-mu(1)
ll1.append resma = res - theta(1)*resma(-1)
ll1.append sig2 = omega(1)+omega(2)*dumw-omega(2)*(alpha(1)+beta(1))*dumw(-1) +alpha(1)*resma(-1)^2 +beta(1)*sig2(-1)
ll1.append z = resma/@sqrt(sig2)
ll1.append logl =  log(@dnorm(z)) - log(sig2)/2


' estimate and display results
smpl s1
ll1.ml(showopts, m=1000, c=1e-5)
show ll1.output
